package ch.unisg.tapas.auctionhouse.adapter.in.web;

import ch.unisg.tapas.auctionhouse.adapter.common.formats.AuctionHouseDiscoveryJsonRepresentation;
import ch.unisg.tapas.auctionhouse.application.port.in.AuctionHouseDiscoveryUseCase;
import ch.unisg.tapas.auctionhouse.application.port.in.AuctionHouseDiscoveryCommand;
import ch.unisg.tapas.auctionhouse.domain.AuctionHouseDiscoveryInformation;
import com.fasterxml.jackson.core.JsonProcessingException;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;


/**
 * Controller that handles HTTP requests for the auction house discovery. This controller implements
 * the {@link AuctionHouseDiscoveryUseCase} use case using the {@link AuctionHouseDiscoveryCommand} command.
 */
@RestController
public class AuctionHouseDiscoveryWebController {
    private final AuctionHouseDiscoveryUseCase auctionHouseDiscoveryUseCase;

    public AuctionHouseDiscoveryWebController(AuctionHouseDiscoveryUseCase auctionHouseDiscoveryUseCase) {
        this.auctionHouseDiscoveryUseCase = auctionHouseDiscoveryUseCase;
    }

    @GetMapping(path="/discovery/", consumes = AuctionHouseDiscoveryJsonRepresentation.MEDIA_TYPE)
    public ResponseEntity<String> auctionHouseDiscovery(@RequestBody AuctionHouseDiscoveryJsonRepresentation payload) {
        AuctionHouseDiscoveryInformation.GroupName groupName = (payload.getGroupName() == null) ?
            null : new AuctionHouseDiscoveryInformation.GroupName(payload.getGroupName());

        AuctionHouseDiscoveryCommand command = new AuctionHouseDiscoveryCommand(
            new AuctionHouseDiscoveryInformation.AuctionHouseUri(payload.getAuctionHouseUri()),
            new AuctionHouseDiscoveryInformation.WebSubUri(payload.getAuctionHouseUri()),
            new AuctionHouseDiscoveryInformation.TaskTypes(payload.getTaskTypes()),
            new AuctionHouseDiscoveryInformation.TimeStamp(payload.getTimeStamp()),
            groupName
        );

        AuctionHouseDiscoveryInformation auctionHouseDiscoveryInformation = auctionHouseDiscoveryUseCase.auctionHouseDiscovery(command);

        try {
            AuctionHouseDiscoveryJsonRepresentation representation = new AuctionHouseDiscoveryJsonRepresentation(auctionHouseDiscoveryInformation);
            String auctionHouseDiscoveryJson = AuctionHouseDiscoveryJsonRepresentation.serialize(auctionHouseDiscoveryInformation);

            HttpHeaders responseHeaders = new HttpHeaders();
            responseHeaders.add(HttpHeaders.CONTENT_TYPE, AuctionHouseDiscoveryJsonRepresentation.MEDIA_TYPE);

            return new ResponseEntity<>(auctionHouseDiscoveryJson, responseHeaders, HttpStatus.OK);
        } catch (JsonProcessingException e) {
            throw new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
}
